SET SERVEROUTPUT ON;
DECLARE
 -- DOM-Struktur
 doc DBMS_XMLDOM.DOMDocument;
 -- Prozessoranweisung
 pi DBMS_XMLDOM.DOMProcessingInstruction; 
 -- Knoten
 n               DBMS_XMLDOM.DOMNode;
 e               DBMS_XMLDOM.DOMElement;
 root            DBMS_XMLDOM.DOMNode;
 kurslisteNode   DBMS_XMLDOM.DOMNode;
 kursNode        DBMS_XMLDOM.DOMNode;
 terminlisteNode DBMS_XMLDOM.DOMNode;
 terminNode      DBMS_XMLDOM.DOMNode;
 t               DBMS_XMLDOM.DOMText; 
 -- Buffer fr Kontrollausgabe
 buffer varchar2(32676);
 -- Cursor fr relationale Daten
 CURSOR c_Termine (b_Titel kurs.K_Titel%TYPE)
 IS
  SELECT K_Titel, K_Untertitel,
         CURSOR(SELECT T_Nr, T_Ort, T_Beginn 
                  FROM termin
                 WHERE termin.K_Nr = kurs.K_Nr) AS Termine
   FROM kurs
  WHERE K_Titel = b_Titel;
 -- Datensatztypen
 TYPE ds_Kurs IS RECORD (
   K_Titel kurs.K_Titel%TYPE,
   K_Untertitel kurs.K_Untertitel%TYPE
 );
 v_KDaten ds_Kurs;
 TYPE ds_Termin IS RECORD (
   T_Nr termin.T_Nr%TYPE,
   T_Ort termin.T_Ort%TYPE,
   T_Beginn termin.T_Beginn%TYPE
 ); 
 v_TDaten ds_Termin;
 -- Cursorvariable
 TYPE cv_TerminTyp IS REF CURSOR;
 cv_Termin cv_TerminTyp;  
 -- Element-Erstellung
 PROCEDURE appendElement(name VARCHAR2,
                         content VARCHAR2,
                         doc IN OUT DBMS_XMLDOM.DOMDocument,
                         parent IN OUT DBMS_XMLDOM.DOMNode)
  IS
   n  DBMS_XMLDOM.DOMNode;
   e  DBMS_XMLDOM.DOMElement;
   t  DBMS_XMLDOM.DOMText; 
  BEGIN
   e := DBMS_XMLDOM.createElement( doc, name );
   n := DBMS_XMLDOM.makeNode( e );
   n := DBMS_XMLDOM.appendChild( parent, n );
   t := DBMS_XMLDOM.createTextNode( doc, content );
   n := DBMS_XMLDOM.appendChild( n, DBMS_XMLDOM.makeNode( t ) ); 
  END;   
BEGIN
 -- Erstelle neues DOM-Dokument
 doc := DBMS_XMLDOM.newDOMDocument();
 -- Prolog
 DBMS_XMLDOM.setVersion(doc, '1.0'); -- Version
 DBMS_XMLDOM.setCharset(doc, 'UTF-8'); -- Zeichensatz
 DBMS_XMLDOM.setStandalone(doc, 'no'); -- standalone?
 -- Wurzelelement erstellen
 root := DBMS_XMLDOM.makeNode( doc );
 e := DBMS_XMLDOM.createElement( doc, 'Kursliste' );
 -- XML Schema-Verknpfung
 DBMS_XMLDOM.setAttribute( e, 'xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance');
 DBMS_XMLDOM.setAttribute( e, 'xsi:noNamespaceSchemaLocation', 'kursliste.xsd');
 n := DBMS_XMLDOM.makeNode( e );
 kurslisteNode := DBMS_XMLDOM.appendChild( root, n );
 -- Abfrage verarbeiten und Kinder erzeugen
 OPEN c_Termine('Oracle');
 LOOP
  FETCH c_Termine INTO v_KDaten.K_Titel, v_KDaten.K_Untertitel, cv_Termin;
  EXIT WHEN c_Termine%NOTFOUND;
  -- Kurs
  e := DBMS_XMLDOM.createElement( doc, 'Kurs' );
  n := DBMS_XMLDOM.makeNode( e );
  kursNode := DBMS_XMLDOM.appendChild( kurslisteNode, n );
  -- K_Titel
  appendElement('K_Titel', v_KDaten.K_Titel, doc, kursNode);
  -- K_Untertitel
  appendElement('K_Untertitel', v_KDaten.K_Untertitel, doc, kursNode);
  -- Terminliste
  e := DBMS_XMLDOM.createElement( doc, 'Terminliste' );
  n := DBMS_XMLDOM.makeNode( e );
  terminlisteNode := DBMS_XMLDOM.appendChild( kursNode, n );  
  LOOP
   FETCH cv_Termin INTO v_TDaten;
   EXIT WHEN cv_Termin%NOTFOUND;
  -- Termin-Element erzeugen
  e := DBMS_XMLDOM.createElement( doc, 'Termin' );
  n := DBMS_XMLDOM.makeNode( e );
  terminNode := DBMS_XMLDOM.appendChild( terminlisteNode, n );
  -- T_Nr
  appendElement('T_Nr', TO_CHAR(v_TDaten.T_Nr), doc, terminNode); 
  -- T_Ort
  appendElement('T_Ort', v_TDaten.T_Ort, doc, terminNode);
  -- T_Beginn
  appendElement('T_Beginn', TO_CHAR(v_TDaten.T_Beginn, 'YYYY-MM-DD'), doc, terminNode);  
 END LOOP; 
 END LOOP;
 CLOSE c_Termine;
 -- Kontrollausgabe
 DBMS_XMLDOM.writeToBuffer(doc, buffer);
 DBMS_OUTPUT.put_line(buffer);
END;
