SET SERVEROUTPUT ON;
DECLARE
 -- Verzeichnis
 dir         VARCHAR2(100) := 'C:\Oraxml';
 -- XML-Datei (extern)
 xmlfile     VARCHAR2(100) := 'dozentenliste.xml'; 
 -- XSLT-Datei (extern)
 xslfile     VARCHAR2(100) := 'dozentenliste2.xslt';
 -- Ergebnisdatei (extern)
 resfile     VARCHAR2(100) := 'dozentenliste2.html'; 
 -- Fehlerdatei (extern)
 errfile     VARCHAR2(100) := 'dozentenliste2.txt';
 
 -- Parser
 p           DBMS_XMLPARSER.Parser; -- Parser
 xmldoc      DBMS_XMLDOM.DOMDocument; -- DOM-Dokument
 xmldocnode  DBMS_XMLDOM.DOMNode; -- DOM-Knoten
 proc        DBMS_XSLPROCESSOR.Processor; -- XSLT-Prozessor
 ss          DBMS_XSLPROCESSOR.Stylesheet; -- XSLT-Datei
 xsldoc      DBMS_XMLDOM.DOMDocument; -- XSLT-DOM-Dokument
 docfrag     DBMS_XMLDOM.DOMDocumentFragment; -- DOM-Fragment
 docfragnode DBMS_XMLDOM.DOMNode; -- DOM-Fragment-Knoten
 xslelem     DBMS_XMLDOM.DOMElement; -- DOM-Element
BEGIN
 -- Erzeuge Parser
 p := DBMS_XMLPARSER.newParser;

 -- Eigenschaften des Parsers
 -- Fehlerdatei
 DBMS_XMLPARSER.setErrorLog(p, dir || '/' || errfile);
 -- Leerzeichenbewahrung
 DBMS_XMLPARSER.setPreserveWhiteSpace(p, TRUE);

 -- Transformation XML->DOM
 DBMS_XMLPARSER.parse(p,  dir || '/' || xmlfile);

 -- DOM-Dokument erhalten
 xmldoc := DBMS_XMLPARSER.getDocument(p);

 -- Transformation XML->HTML
 DBMS_XMLPARSER.parse(p, dir || '/' || xslfile);

 -- DOM-Dokument von XSLT-Datei erhalten
 xsldoc := DBMS_XMLPARSER.getDocument(p);
 -- Angabe der XSLT-Datei
 ss := DBMS_XSLPROCESSOR.newStylesheet(xsldoc, dir || '/' || xslfile);
 DBMS_XSLPROCESSOR.setParam(ss, 'Adressausgabe', '''Zeile''');
 -- Transformation XML mit XSLT -> HTML
 proc := DBMS_XSLPROCESSOR.newProcessor;
 DBMS_XSLPROCESSOR.showWarnings(proc, true);
 DBMS_XSLPROCESSOR.setErrorLog(proc, dir || '/' || errfile);
 
 docfrag := DBMS_XSLPROCESSOR.processXSL(proc, ss, xmldoc);
 docfragnode := DBMS_XMLDOM.makeNode(docfrag);
 -- Dateierstellung
 DBMS_XMLDOM.writeToFile(docfragnode, dir || '/' || resfile); 

 -- XML_Dateien lsen
 DBMS_XMLDOM.freeDocument(xmldoc);
 DBMS_XMLDOM.freeDocument(xsldoc);

 -- XSL-Datei und Prozessor entfernen
 DBMS_XSLPROCESSOR.freeProcessor(proc);
 DBMS_XSLPROCESSOR.freeStylesheet(ss);

 -- Parser entfernen
 DBMS_XMLPARSER.freeParser(p);
END;