-- XML Schema ggf. lschen
BEGIN
 DBMS_XMLSCHEMA.DELETESCHEMA(
  schemaurl     => 'http://www.kurse-nrw.de/kurs.xsd',
  delete_option => DBMS_XMLSCHEMA.DELETE_CASCADE);
END;

-- Schema aufgrund von XMLType registrieren
BEGIN
 DBMS_XMLSCHEMA.registerSchema(
  schemaurl => 'http://www.kurse-nrw.de/kurs.xsd',
  schemadoc => XMLType('<?xml version="1.0"?>
<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xdb="http://xmlns.oracle.com/xdb" xsi:schemaLocation="http://xmlns.oracle.com/xdb http://xmlns.oracle.com/xdb/XDBSchema.xsd">
 <xsd:element name="Kurs" type="Kurs_TType" xdb:global="true" xdb:SQLType="Kurs_T" xdb:SQLSchema="SCOTT" xdb:defaultTable="Kurs_x"/>
 <xsd:complexType name="Kurs_TType" xdb:SQLType="Kurs_T" xdb:SQLSchema="SCOTT" xdb:maintainDOM="false">
  <xsd:sequence>
   <xsd:element name="Titel" xdb:SQLName="Titel" xdb:SQLType="VARCHAR2">
    <xsd:simpleType>
     <xsd:restriction base="xsd:string">
      <xsd:maxLength value="40"/>
     </xsd:restriction>
    </xsd:simpleType>
   </xsd:element>
   <xsd:element name="Untertitel" xdb:SQLName="Untertitel" xdb:SQLType="VARCHAR2">
    <xsd:simpleType>
     <xsd:restriction base="xsd:string">
      <xsd:maxLength value="40"/>
     </xsd:restriction>
    </xsd:simpleType>
   </xsd:element>
   <xsd:element name="Dauer" type="xsd:double" xdb:SQLName="Dauer" xdb:SQLType="NUMBER"/>
   <xsd:element name="Nr" xdb:SQLName="Nr" xdb:SQLType="VARCHAR2">
    <xsd:simpleType>
     <xsd:restriction base="xsd:string">
      <xsd:maxLength value="30"/>
     </xsd:restriction>
    </xsd:simpleType>
   </xsd:element>
  </xsd:sequence>
 </xsd:complexType>
</xsd:schema>'),
 local  => TRUE, 
 gentypes => TRUE, 
 genbean => FALSE,
 force => FALSE,
 owner  => 'SCOTT',
 enablehierarchy  => DBMS_XMLSCHEMA.ENABLE_HIERARCHY_CONTENTS);
END;

-- Tabelle auf Basis des XML Schemas erstellen
/*
DROP TABLE kurs_x1;
DROP TABLE kurs_x2;
DROP TABLE kurs_x3;
DROP TABLE kurs_x4;
*/

-- XMLType-Tabelle
CREATE TABLE kurs_x1 OF XMLTYPE
  XMLSCHEMA "http://www.kurse-nrw.de/kurs.xsd"
  ELEMENT "Kurs";
-- XMLType-Spalte  
CREATE TABLE kurs_x2 (K_Nr NUMBER, K_Inhalt XMLType)
  XMLTYPE COLUMN K_Inhalt
  ELEMENT
    "http://www.kurse-nrw.de/kurs.xsd#Kurs";
-- XMLType-Tabelle mit CLOB-Speicherung    
CREATE TABLE kurs_x3 OF XMLType
  XMLTYPE STORE AS CLOB
  XMLSCHEMA "http://www.kurse-nrw.de/kurs.xsd"
  ELEMENT "Kurs";
-- XMLType-Spalte mit CLOB-Speicherung 
CREATE TABLE kurs_x4 (K_Nr NUMBER, K_Inhalt XMLType)
  XMLTYPE COLUMN K_Inhalt
  STORE AS CLOB
  XMLSCHEMA "http://www.kurse-nrw.de/kurs.xsd"
  ELEMENT "Kurs";
   
-- Daten einfgen
INSERT INTO kurs_x1
SELECT SYS_XMLGEN("Kurs_T"(K_Titel,
                           K_Untertitel,
                           K_Nr,
                           K_Dauer),
       XMLFormat.createFormat('Kurs')) AS XML
 FROM kurs
WHERE K_Titel = 'Oracle'; 

INSERT INTO kurs_x2
SELECT K_Nr, 
       SYS_XMLGEN("Kurs_T"(K_Titel,
                           K_Untertitel,
                           K_Nr,
                           K_Dauer),
       XMLFormat.createFormat('Kurs')) AS XML
 FROM kurs
WHERE K_Titel = 'Oracle'; 

INSERT INTO kurs_x3
SELECT SYS_XMLGEN("Kurs_T"(K_Titel,
                           K_Untertitel,
                           K_Nr,
                           K_Dauer),
       XMLFormat.createFormat('Kurs')) AS XML
 FROM kurs
WHERE K_Titel = 'Oracle'; 

INSERT INTO kurs_x4
SELECT K_Nr, 
       SYS_XMLGEN("Kurs_T"(K_Titel,
                           K_Untertitel,
                           K_Nr,
                           K_Dauer),
       XMLFormat.createFormat('Kurs')) AS XML
 FROM kurs
WHERE K_Titel = 'Oracle'; 
 
-- Test
DECLARE
 v_XML XMLType;
 v_KNr NUMBER;
BEGIN
 SELECT *
   INTO v_XML
   FROM kurs_x1
  WHERE ROWNUM = 1;
 DBMS_OUTPUT.PUT_LINE(v_XML.getStringVal());
 
 SELECT K_Nr, K_Inhalt
   INTO v_KNR, v_XML
   FROM kurs_x2
  WHERE ROWNUM = 1;
 DBMS_OUTPUT.PUT_LINE(v_XML.getStringVal());  
END;

/*
<Kurs>
  <Obertitel>Oracle</Obertitel>
  <Untertitel>Architektur</Untertitel>
  <Tage>1025051</Tage>
  <Kursnummer>3</Kursnummer>
</Kurs>
*/


