CREATE OR REPLACE  PACKAGE BODY "SCOTT"."GENERATOR" as
-------------------Zufall
FUNCTION BerechneZufall (
 a        IN NUMBER,
 r        IN NUMBER,
 c        IN NUMBER,
 m        IN NUMBER )
 RETURN NUMMERN
IS
 t_Zufall NUMMERN;
 r2 NUMBER := r;
BEGIN
 FOR i IN 1..m LOOP
  r2 := (a*r2 + c)mod m;
  IF r2 = 0 THEN r2 := 1; END IF;
  t_Zufall(i) := r2;
 END LOOP;
 RETURN t_Zufall;
END BerechneZufall;
-------------------SQL-Funktionen
-- ErmittlePreis ermittelt Gruppenpreis
FUNCTION ErmittlePreis (
 Terminnummer   IN termin.T_Nr%TYPE,
 Teilnehmerzahl IN buchung.B_TZahl%TYPE)
RETURN buchung.B_Preis%TYPE
IS
 v_KPreis preis.P_TN1%TYPE;     -- Kurspreis
 v_SQL    VARCHAR2(300);        -- Abfrage
 v_TNZahl buchung.B_TZahl%TYPE; -- TN-Zahl
 v_KNr    kurs.K_Nr%TYPE;       -- Kursnr
BEGIN
 -- Untersuchung der Eingaben
 IF    Teilnehmerzahl > 5  THEN v_TNZahl := 5;
 ELSIF Teilnehmerzahl <= 0 THEN v_TNZahl := 1;
 ELSE v_TNZahl := Teilnehmerzahl;
 END IF;
 -- Ermittlung der Kursnummer
 SELECT K_Nr
   INTO v_KNr
   FROM kurs NATURAL JOIN termin
  WHERE T_Nr = Terminnummer;
 -- Abfrage fr Preisbestimmung
 v_SQL := 'SELECT P_TN'
          || v_TNZahl
          || ' FROM kurs NATURAL JOIN preis'
          || ' WHERE K_Nr='
          || v_Knr;
 EXECUTE IMMEDIATE v_SQL
 INTO v_KPreis;
 RETURN v_KPreis;
 EXCEPTION
 WHEN NO_DATA_FOUND OR TOO_MANY_ROWS  OR VALUE_ERROR
  THEN v_KPreis := 0; RETURN v_KPreis;
END ErmittlePreis;

-- ErmittleBereich ermittelt K_Bereich fr T_Nr
FUNCTION ErmittleBereich (
 Terminnummer   IN termin.T_Nr%TYPE)
RETURN kurs.K_Bereich%TYPE
IS
 v_KBereich kurs.K_Bereich%TYPE;
BEGIN
 SELECT K_Bereich
   INTO v_KBereich
   FROM termin NATURAL JOIN kurs
  WHERE T_Nr = Terminnummer;
  RETURN v_KBereich;
 EXCEPTION
 WHEN NO_DATA_FOUND OR TOO_MANY_ROWS  OR VALUE_ERROR
  THEN v_KBereich := 'NULL'; RETURN v_KBereich;
END ErmittleBereich;

-- ErmittleDatum ermittelt fr Termin ein Anmeldedatum
FUNCTION ErmittleDatum (
 Terminnummer   IN termin.T_Nr%TYPE,
 Teilnehmerzahl IN buchung.B_TZahl%TYPE)
RETURN buchung.B_Datum%TYPE
IS
 v_BDatum buchung.B_Datum%TYPE;
BEGIN
 SELECT T_Beginn-10*Teilnehmerzahl
   INTO v_BDatum
   FROM termin
  WHERE T_Nr = Terminnummer;
  RETURN v_BDatum;
 EXCEPTION
 WHEN NO_DATA_FOUND OR TOO_MANY_ROWS  OR VALUE_ERROR
  THEN v_BDatum := 'NULL'; RETURN v_BDatum;
END ErmittleDatum;

-- ZaehleReihen ermittelt die passende Buchungsnummer
 FUNCTION ZaehleReihen(
   Tabelle IN  VARCHAR2 := 'buchung',
   Spalte  IN  VARCHAR2 := 'B_Nr')
 RETURN buchung.B_Nr%TYPE
IS
 v_SQL    VARCHAR2(300);
 v_Return buchung.B_Nr%TYPE;
BEGIN
  v_SQL := 'SELECT MAX('
          || Spalte
          || ') FROM '
          || Tabelle;
 EXECUTE IMMEDIATE v_SQL
 INTO v_Return;
 IF v_Return IS NULL
  THEN v_Return := 1;
 END IF;
 RETURN v_Return+1;
END ZaehleReihen;   

---------------Funktionen
-- FindeGruppe findet weitere Anmelder
FUNCTION FindeGruppe (
 Teilnehmernr IN teilnehmer.TN_Nr%TYPE)
RETURN NUMMERN
IS
 t_TNNr NUMMERN;
 v_UNr  teilnehmer.U_Nr%TYPE;
BEGIN
 -- Test auf Unternehmenskunde
 SELECT U_Nr
   INTO v_UNr
   FROM teilnehmer
  WHERE TN_Nr = Teilnehmernr;
 -- Fallunterscheidung
 -- 1. Unternehmenskunde
 IF v_UNr IS NOT NULL THEN
  SELECT TN_Nr
    BULK COLLECT INTO t_TNNr
     FROM teilnehmer
    WHERE U_Nr = (SELECT U_Nr
                    FROM teilnehmer
                   WHERE TN_Nr = Teilnehmernr);
 -- 2. Privatkunde
 ELSE
  SELECT TN_Nr
    BULK COLLECT INTO t_TNNr
    FROM teilnehmer
   WHERE TN_PLZ BETWEEN (SELECT TN_PLZ-1000
                           FROM teilnehmer
                           WHERE TN_Nr = Teilnehmernr)
                AND (SELECT TN_PLZ+1000
                       FROM teilnehmer
                      WHERE TN_Nr = Teilnehmernr);
 END IF;
 -- Test auf zu groe Gruppe (max 6)
 IF t_TNNr.COUNT >6
  THEN t_TNNr.DELETE(7, t_TNNr.COUNT);
 END IF;
 RETURN t_TNNr;                 
 EXCEPTION
 WHEN NO_DATA_FOUND
  THEN t_TNNr(1) := Teilnehmernr; RETURN t_TNNr;
END FindeGruppe;

--------------------------Prozeduren
--BucheTeilnehmer erfasst einzelne Anmeldung
PROCEDURE BucheTeilnehmer (
 Teilnehmernr   IN  buchung.TN_Nr%TYPE,
 Terminnummer   IN  buchung.T_Nr%TYPE,
 Teilnehmerzahl IN  buchung.B_TZahl%TYPE,
 Anmelder       IN  buchung.B_NrAnmelder%TYPE,
 Buchungsnr     OUT buchung.B_Nr%TYPE)
IS
 v_SQL VARCHAR2(500);
BEGIN
 INSERT INTO buchung
 VALUES (ZaehleReihen(),
         Teilnehmernr,
         Terminnummer,
         TO_DATE(ErmittleDatum(Terminnummer, Teilnehmerzahl)),
         Teilnehmerzahl,
         ErmittlePreis(Terminnummer, Teilnehmerzahl),
         Anmelder,
         NULL,
         NULL)
 RETURNING B_Nr INTO Buchungsnr;
END BucheTeilnehmer;

-- 
PROCEDURE Ausfuehrung (
 Anzahl IN NUMBER)
IS
 t_Zufall NUMMERN := BerechneZufall(2,211,482,ZaehleReihen('termin', 'T_Nr')-1);
 -- Termindatenstruktur
 TYPE RecTermin IS RECORD (
  Monat   VARCHAR2(2),
  Bereich kurs.K_Bereich%TYPE 
  );
 v_TDaten RecTermin;
 v_TGewicht NUMBER(4,2);
 v_KGewicht NUMBER(4,2);
 t_TGewicht NUMMERN;
 -- Teilnehmerdatenstruktur
 TYPE RecTeilnehmer IS RECORD (
  Nr          teilnehmer.TN_Nr%TYPE,
  UNr         teilnehmer.U_Nr%TYPE,
  PLZ         teilnehmer.TN_PLZ%TYPE
  );
 v_TNDaten    RecTeilnehmer;
 t_Teilnehmer NUMMERN;
 -- Buchungsergebnis
 v_BNr        buchung.B_Nr%TYPE;
BEGIN
 -- Gewichtungsberechnung: Termine
 FOR i IN 1..ZaehleReihen('termin', 'T_Nr')-1 LOOP
  ------------------
  -- Terminbewertung
  SELECT ErmittleBereich(T_Nr), TO_CHAR(T_Beginn, 'MM')
    INTO v_TDaten.Bereich, v_TDaten.Monat
    FROM termin
   WHERE T_Nr = i;
  -- Monat
  <<Monatsberwertung>>
  CASE v_TDaten.Monat
   WHEN '01' THEN v_TGewicht := t_Monate(1);
   WHEN '02' THEN v_TGewicht := t_Monate(2);
   WHEN '03' THEN v_TGewicht := t_Monate(3);
   WHEN '04' THEN v_TGewicht := t_Monate(4);
   WHEN '05' THEN v_TGewicht := t_Monate(5);
   WHEN '06' THEN v_TGewicht := t_Monate(6);
   WHEN '07' THEN v_TGewicht := t_Monate(7);
   WHEN '08' THEN v_TGewicht := t_Monate(8);
   WHEN '09' THEN v_TGewicht := t_Monate(9);
   WHEN '10' THEN v_TGewicht := t_Monate(10);
   WHEN '11' THEN v_TGewicht := t_Monate(11);
   WHEN '12' THEN v_TGewicht := t_Monate(12);
  END CASE Monatsbewertung;
  -- Bereich
  <<Bereichsbewertung>>
  CASE v_TDaten.Bereich
   WHEN 'Programmierung'         THEN v_KGewicht := t_Kurse(1);
   WHEN 'Datenbanken'            THEN v_KGewicht := t_Kurse(2);
   WHEN 'Programmierung(Office)' THEN v_KGewicht := t_Kurse(3);
   WHEN 'Office'                 THEN v_KGewicht := t_Kurse(4);
   WHEN 'Server'                 THEN v_KGewicht := t_Kurse(5);
   WHEN 'Webdesign'              THEN v_KGewicht := t_Kurse(6);
   WHEN 'Grafik'                 THEN v_KGewicht := t_Kurse(7);
  END CASE Bereichsbewertung;
  -- Gewichtungsberechnung
  t_TGewicht(i) := v_TGewicht*v_KGewicht*t_Zufall(i);
  -- Regel: Verarbeitung nur, wenn Gewicht > Schwellwert
  IF t_TGewicht(i) >= 50 THEN
   SELECT TN_Nr, U_Nr, TN_PLZ
     INTO v_TNDaten
     FROM teilnehmer
    WHERE TN_Nr = i;
   -- Gruppenbildung
   t_Teilnehmer := (FindeGruppe(i));
   FOR j IN t_Teilnehmer.FIRST..t_Teilnehmer.LAST LOOP
    BucheTeilnehmer (
        Teilnehmernr   => t_Teilnehmer(j),
        Terminnummer   => i,
        Teilnehmerzahl => t_Teilnehmer.COUNT,
        Anmelder       => t_Teilnehmer(t_Teilnehmer.FIRST),
        Buchungsnr     => v_BNr);
    COMMIT;
   END LOOP;
   -- Eintrag pro Gruppenmitglied
  END IF;
  EXIT WHEN i = Anzahl;
 END LOOP;
END Ausfuehrung;
end;