-- Tabelle Kurs
--
DROP TABLE "SCOTT"."KURS";
CREATE TABLE "SCOTT"."KURS"
("K_NR"            NUMBER(7)     NOT NULL,
 "K_BEREICH"       VARCHAR2(30)  NOT NULL,
 "K_TITEL"         VARCHAR2(30)  NOT NULL, 
 "K_UNTERTITEL"    VARCHAR2(40)  NOT NULL,
 "K_DAUER"         NUMBER(1)     NOT NULL, 
 "P_NR"            NUMBER(2)     NOT NULL,
 "K_VORKENNTNISSE" VARCHAR2(150) NOT NULL,
 "K_ZIELGRUPPE"    VARCHAR2(50)  NOT NULL,
 "K_THEMEN"        VARCHAR2(600) NOT NULL,
  PRIMARY KEY("K_NR"),
  FOREIGN KEY("P_NR")
    REFERENCES "SCOTT"."PREIS"("P_NR"),   
  CHECK(K_Dauer>0))  
  TABLESPACE "USERS";
--
--Tabelle Termin
--
DROP TABLE "SCOTT"."TERMIN";
CREATE TABLE "SCOTT"."TERMIN"
("T_NR"     NUMBER(4) NOT NULL,
 "K_NR"     NUMBER(7) NOT NULL,
 "T_BEGINN" DATE      NOT NULL,
 "T_ENDE"   DATE      NOT NULL,
 "T_ORT" VARCHAR2(20) NOT NULL,
  PRIMARY KEY("T_NR"),
  FOREIGN KEY("K_NR") 
    REFERENCES "SCOTT"."KURS"("K_NR"),   CHECK(T_Beginn<=T_Ende))  
  TABLESPACE "USERS";
--
--Tabelle Preis
--
DROP TABLE "SCOTT"."PREIS";
CREATE TABLE "SCOTT"."PREIS"
("P_NR"  NUMBER(2)    NOT NULL,
 "P_TN1" NUMBER(6, 2) NOT NULL,
 "P_TN2" NUMBER(6, 2) NOT NULL,
 "P_TN3" NUMBER(6, 2) NOT NULL,
 "P_TN4" NUMBER(6, 2) NOT NULL,
 "P_TN5" NUMBER(6, 2) NOT NULL,
  PRIMARY KEY("P_NR"), 
  CHECK(P_TN1>P_TN2),
  CHECK(P_TN2>P_TN3),
  CHECK(P_TN3>P_TN4), 
  CHECK(P_TN4>P_TN5))  
  TABLESPACE "USERS";
--
-- Tabelle Unternehmen
--
CREATE TABLE "SCOTT"."UNTERNEHMEN"
("U_NR"        NUMBER(5) NOT NULL, 
 "U_NAME"      VARCHAR2(200),
 "U_BRANCHE"   VARCHAR2(100), 
 "U_STRASSE"   VARCHAR2(30),
 "U_HAUSNR"    VARCHAR2(5), 
 "U_STADTTEIL" VARCHAR2(30),
 "U_LAND"      VARCHAR2(2),
 "U_PLZ"       NUMBER(5),
 "U_STADT"     VARCHAR2(30),
 "U_TELART"    VARCHAR2(10), 
 "U_VORWAHL"   NUMBER(10),
 "U_TELNR"     NUMBER(10),
  PRIMARY KEY("U_NR"))  
  TABLESPACE "USERS";
--
-- Tabelle Teilnehmer
--
DROP TABLE "SCOTT"."TEILNEHMER";
CREATE TABLE "SCOTT"."TEILNEHMER"
("TN_NR"        NUMBER(4)    NOT NULL,
 "U_NR"         NUMBER(5),
 "TN_ANREDE"    VARCHAR2(5),
 "TN_VORNAME"   VARCHAR2(20), 
 "TN_NACHNAME"  VARCHAR2(30),
 "TN_STRASSE"   VARCHAR2(40), 
 "TN_HAUSNR"    VARCHAR2(5),
 "TN_STADTTEIL" VARCHAR2(30), 
 "TN_LAND"      VARCHAR2(2),
 "TN_PLZ"       NUMBER(5),
 "TN_STADT"     VARCHAR2(30),
 "TN_VORWAHL"   NUMBER(10),
 "TN_TELNR"     NUMBER(10),
  PRIMARY KEY("TN_NR"),
  FOREIGN KEY("U_NR")
    REFERENCES "SCOTT"."UNTERNEHMEN"("U_NR"))  
  TABLESPACE "USERS";
--
-- Tabelle Dozent
--
DROP TABLE "SCOTT"."DOZENT";
CREATE TABLE "SCOTT"."DOZENT"
("D_NR"        NUMBER(4) NOT NULL, 
 "D_ANREDE"    VARCHAR2(5),
 "D_VORNAME"   VARCHAR2(20), 
 "D_NACHNAME"  VARCHAR2(30),
 "D_STRASSE"   VARCHAR2(40), 
 "D_HAUSNR"    VARCHAR2(5),
 "D_STADTTEIL" VARCHAR2(30), 
 "D_LAND"      VARCHAR2(2),
 "D_PLZ"       NUMBER(5),
 "D_STADT"     VARCHAR2(30),
 "D_VORWAHL"   NUMBER(10),
 "D_TELNR"     NUMBER(10),
  PRIMARY KEY("D_NR"))  
  TABLESPACE "USERS";
--
-- Tabelle Themenverteilung
--
DROP TABLE "SCOTT"."THEMENVERTEILUNG";
CREATE TABLE "SCOTT"."THEMENVERTEILUNG"
("TH_NR"        NUMBER(4)    NOT NULL,
 "D_NR"         NUMBER(4)    NOT NULL,
 "K_NR"         NUMBER(7)    NOT NULL, 
 "TH_TAGESSATZ" NUMBER(6, 2) NOT NULL,
  PRIMARY KEY("TH_NR"), 
  FOREIGN KEY("D_NR") 
    REFERENCES "SCOTT"."DOZENT"("D_NR"),
  FOREIGN KEY("K_NR") 
    REFERENCES "SCOTT"."KURS"("K_NR"))  
 TABLESPACE "USERS";
--
-- Tabelle Buchung
--
DROP TABLE "SCOTT"."BUCHUNG";
CREATE TABLE "SCOTT"."BUCHUNG" (
 "B_NR"            NUMBER(6)      NOT NULL ,
 "TN_NR"           NUMBER(4),
 "T_NR"            NUMBER(4),
 "B_DATUM"         DATE,
 "B_TZAHL"         NUMBER(1),
 "B_PREIS"         NUMBER(10, 2),
 "B_NRANMELDER"    NUMBER(5),
 "B_STORNODATUM"   DATE,
 "B_STORNOGEBUEHR" NUMBER(10, 2),
  PRIMARY KEY ( "B_NR" ),
  FOREIGN KEY("TN_NR") 
   REFERENCES "SCOTT"."TEILNEHMER"("TN_NR"),
  FOREIGN KEY("T_NR") 
   REFERENCES "SCOTT"."TERMIN"("T_NR"))
 TABLESPACE "USERS";
--
-- Alle Tabelle lschen
--
/*
DROP TABLE buchung;
DROP TABLE themenverteilung;
DROP TABLE termin;
DROP TABLE kurs;
DROP TABLE preis;
DROP TABLE dozent;
DROP TABLE teilnehmer;
DROP TABLE unternehmen;
*/